@@ -1,5 +1,5 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/functions.c,v 1.56 2004/10/15 19:20:48 sampie Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/functions.c,v 1.58 2004/11/11 19:22:41 richb Exp $
  *
  *  Copyright (c) 1987-2004 Sun Microsystems, Inc. All Rights Reserved.
  *           
@@ -473,7 +473,7 @@ do_calc()      /* Perform arithmetic calculation and display result. */
         mpmul(v->MPresult, MP1, v->MPresult);
 
     } else if (IS_KEY(v->cur_op, KEY_YTOX.value[0])) {    /* y^x */
-        mppwr2(v->MPresult, v->MPdisp_val, v->MPresult);
+        calc_xpowy(v->MPresult, v->MPdisp_val, v->MPresult);
 
     } else if (IS_KEY(v->cur_op, KEY_AND.value[0])) {     /* And */
         mpcmd(v->MPresult, &dres);
@@ -1339,6 +1339,13 @@ save_pending_values(struct button *but)
 }
 
 
+void
+show_error(char *message)
+{
+    update_statusbar(message, "gtk-dialog-error");
+}
+
+
 static struct button *
 ch_to_button(struct button buttons[], int max_buttons, char ch)
 {
@@ -1374,7 +1381,7 @@ button_for_value(int val)
 {
     struct button *b;
 
-    if ((b = val_to_button(b_buttons, B_NOBUTTONS, val)) == NULL) {
+    if ((b = val_to_button(a_buttons, A_NOBUTTONS, val)) == NULL) {
         if ((b = val_to_button(f_buttons, F_NOBUTTONS, val)) == NULL) {
             b = val_to_button(s_buttons, S_NOBUTTONS, val);
         }
@@ -1389,7 +1396,7 @@ button_for_fc(char ch)
 {
     struct button *b;
 
-    if ((b = ch_to_button(b_buttons, B_NOBUTTONS, ch)) == NULL) {
+    if ((b = ch_to_button(a_buttons, A_NOBUTTONS, ch)) == NULL) {
         if ((b = ch_to_button(f_buttons, F_NOBUTTONS, ch)) == NULL) {
             b = ch_to_button(s_buttons, S_NOBUTTONS, ch);
         }
